# Copyright (c) 2022, NVIDIA CORPORATION.  All Rights Reserved.
#
# NVIDIA CORPORATION and its licensors retain all intellectual property
# and proprietary rights in and to this software, related documentation
# and any modifications thereto.  Any use, reproduction, disclosure or
# distribution of this software and related documentation without an express
# license agreement from NVIDIA CORPORATION is strictly prohibited.
#

from tkinter import StringVar
from tkinter.ttk import Label, LabelFrame

from ...utils.common import PAttrs, PNames, err_symbol
from ...utils.layoutmanager import fill_parent, grid_span, padding
from ...utils.subjectobserver import Observer

class SensorFrame(LabelFrame, Observer):
    def __init__(self, master=None, **kwargs):
        super().__init__(master, text="Sensor", **kwargs)
        self.create_widgets()
        self.manage_layout()
        fill_parent(self)

    def create_widgets(self):
        # warn user that current temperature is too close to sw shutdown limit
        self.WARN_RANGE_IN_C = 2.5

        self.sensor_names = PNames.THERMAL_SENSOR_NAMES
        self.sensor_nums = len(self.sensor_names)
        self.sensor_l = [None] * self.sensor_nums
        self.temp_l = [None] * self.sensor_nums
        self.temp_v = [None] * self.sensor_nums
        self.name_hl = Label(self, text="Name")
        self.temperature_hl = Label(self, text="Temperature (C)")

        for i in range(self.sensor_nums):
            self.temp_v[i] = StringVar()
            self.sensor_l[i] = Label(self, text=self.sensor_names[i])
            self.temp_l[i] = Label(self, textvariable=self.temp_v[i])

    def manage_layout(self):
        self.name_hl.grid(row=0, column=0, **grid_span, **padding)
        self.temperature_hl.grid(row=0, column=1, **grid_span, **padding)

        for i in range(self.sensor_nums):
            self.sensor_l[i].grid(row=1 + i, column=0, **grid_span, **padding)
            self.temp_l[i].grid(row=1 + i, column=1, **grid_span, **padding)

    def update_data(self, data):
        for i in range(self.sensor_nums):
            sensor_name = self.sensor_names[i]
            curr_temp = data[(sensor_name, PAttrs.CURR_TEMP)]
            s_th_temp = data[(sensor_name, PAttrs.SW_THROTTLING_TEMP)]
            s_sd_temp = data[(sensor_name, PAttrs.SW_SHUTDOWN_TEMP)]

            postfix = " C" if curr_temp != err_symbol else ""

            # Only check thermal zones that have software throttling limit
            if err_symbol not in (curr_temp, s_th_temp):
                if curr_temp >= s_th_temp:
                    postfix = " C (Software throttling!)"

            # Only check thermal zones that have software shutdown limit
            if err_symbol not in (curr_temp, s_sd_temp):
                if curr_temp >= s_sd_temp - self.WARN_RANGE_IN_C:
                    postfix = " C (Close to software shutdown limit!)"
                if curr_temp > s_sd_temp:
                    postfix = " C (Software shutdown!)"

            self.temp_v[i].set(f"{curr_temp}{postfix}")