# Copyright (c) 2022, NVIDIA CORPORATION.  All Rights Reserved.
#
# NVIDIA CORPORATION and its licensors retain all intellectual property
# and proprietary rights in and to this software, related documentation
# and any modifications thereto.  Any use, reproduction, disclosure or
# distribution of this software and related documentation without an express
# license agreement from NVIDIA CORPORATION is strictly prohibited.
#

from tkinter.ttk import Frame

from ..utils.datacenter import data_center
from ..utils.layoutmanager import fill_parent, frame_style, padding
from .components import (
    ThermalFrame,
    FanFrame,
    PowerMonitorFrame,
)

class ThermalPowerMonitor(Frame):
    def __init__(self, master=None, **kwargs):
        super().__init__(master, **kwargs)
        self.create_widgets()
        self.attach_observers()
        self.manage_layout()
        fill_parent(self)

    def create_widgets(self):
        self.thermal_f = ThermalFrame(self, **frame_style)
        self.fan1_f = FanFrame(self, fan_id=1, **frame_style)
        if data_center.fan_nums == 2: # TODO: Assume at most 2 fans
            self.fan2_f = FanFrame(self, fan_id=2, **frame_style)

        self.power_monitor_f = PowerMonitorFrame(self, **frame_style)

    def attach_observers(self):
        # these frames need to know power data once updated,
        # so they are seen as observers of the data center
        data_center.attach(self.thermal_f)
        data_center.attach(self.fan1_f)
        if data_center.fan_nums == 2: # TODO: Assume at most 2 fans
            data_center.attach(self.fan2_f)

        data_center.attach(self.power_monitor_f)

    def manage_layout(self):
        heading_row = 1  # if a frame has heading, it will occupy 1 row

        # Predefine how many rows are there in each frame
        sensor_rows = heading_row + data_center.sensor_nums
        fan_rows = 2
        power_rows = heading_row + data_center.rail_nums

        next_starting_row = 0
        self.thermal_f.grid(row=next_starting_row,
                            column=0,
                            rowspan=sensor_rows,
                            columnspan=6,
                            # columnspan = 4 should be enough, but 6 (same with
                            # power frame below) looks good
                            sticky="WENS",
                            **padding)
        next_starting_row += sensor_rows
        self.fan1_f.grid(row=next_starting_row,
                         column=0,
                         rowspan=fan_rows,
                         columnspan=3,
                         sticky="WENS",
                         **padding)
        if data_center.fan_nums == 2:  # TODO: Assume at most 2 fans
            self.fan2_f.grid(row=next_starting_row,
                             column=3,
                             rowspan=fan_rows,
                             columnspan=3,
                             sticky="WENS",
                             **padding)
        next_starting_row += fan_rows
        self.power_monitor_f.grid(row=next_starting_row,
                                  column=0,
                                  rowspan=power_rows,
                                  columnspan=6,
                                  sticky="WENS",
                                  **padding)